@@ -16,6 +16,7 @@
 Code related to interfacing with a Neural Network in the DeepSparse Engine using python
 """
 
+import logging
 import time
 from enum import Enum
 from typing import Dict, Iterable, List, Optional, Tuple, Union
@@ -56,11 +57,15 @@ __all__ = [
     "benchmark_model",
     "analyze_model",
     "Scheduler",
+    "Context",
+    "MultiModelEngine",
 ]
 
+_LOGGER = logging.getLogger(__name__)
 
 ARCH = cpu_architecture()
 NUM_CORES = ARCH.num_available_physical_cores
+NUM_STREAMS = 0  # Default num-streams. Actual value is scheduler-dependent.
 AVX_TYPE = ARCH.isa
 VNNI = ARCH.vnni
 
@@ -116,6 +121,22 @@ def _validate_num_cores(num_cores: Union[None, int]) -> int:
     return num_cores
 
 
+def _validate_num_streams(num_streams: Union[None, int], num_cores: int) -> int:
+    if not num_streams:
+        num_streams = NUM_STREAMS
+
+    if num_streams < 0:
+        raise ValueError("num_streams must be greater than 0")
+
+    if num_streams > num_cores:
+        num_streams = num_cores
+        _LOGGER.warn(
+            "num_streams exceeds num_cores - capping to {}".format(num_streams)
+        )
+
+    return num_streams
+
+
 def _validate_scheduler(scheduler: Union[None, str, Scheduler]) -> Scheduler:
     if not scheduler:
         scheduler = Scheduler.default
@@ -150,6 +171,8 @@ class Engine(object):
     :param num_cores: The number of physical cores to run the model on. If more
         cores are requested than are available on a single socket, the engine
         will try to distribute them evenly across as few sockets as possible.
+    :param num_streams: The max number of requests the model can handle
+        concurrently.
     :param scheduler: The kind of scheduler to execute with. Pass None for the default.
     :param input_shapes: The list of shapes to set the inputs to. Pass None to use model as-is.
     """
@@ -158,13 +181,15 @@ class Engine(object):
         self,
         model: Union[str, Model, File],
         batch_size: int,
-        num_cores: int,
+        num_cores: int = None,
+        num_streams: int = None,
         scheduler: Scheduler = None,
         input_shapes: List[List[int]] = None,
     ):
         self._model_path = model_to_path(model)
         self._batch_size = _validate_batch_size(batch_size)
         self._num_cores = _validate_num_cores(num_cores)
+        self._num_streams = _validate_num_streams(num_streams, self._num_cores)
         self._scheduler = _validate_scheduler(scheduler)
         self._input_shapes = input_shapes
         self._cpu_avx_type = AVX_TYPE
@@ -178,14 +203,18 @@ class Engine(object):
                     model_path,
                     self._batch_size,
                     self._num_cores,
+                    self._num_streams,
                     self._scheduler.value,
+                    None,
                 )
         else:
             self._eng_net = LIB.deepsparse_engine(
                 self._model_path,
                 self._batch_size,
                 self._num_cores,
+                self._num_streams,
                 self._scheduler.value,
+                None,
             )
 
     def __call__(
@@ -251,6 +280,14 @@ class Engine(object):
         """
         return self._num_cores
 
+    @property
+    def num_streams(self) -> int:
+        """
+        :return: The max count of streams the current instance can handle
+           concurrently.
+        """
+        return self._num_streams
+
     @property
     def scheduler(self) -> Scheduler:
         """
@@ -515,16 +552,119 @@ class Engine(object):
             "onnx_file_path": self.model_path,
             "batch_size": self.batch_size,
             "num_cores": self.num_cores,
+            "num_streams": self.num_streams,
             "scheduler": self.scheduler,
             "cpu_avx_type": self.cpu_avx_type,
             "cpu_vnni": self.cpu_vnni,
         }
 
 
+class Context(object):
+    """
+    Contexts can be used to run multiple instances of the MultiModelEngine with the same
+    scheduler. This allows one scheduler to manage the resources of the system
+    effectively, keeping engines that are running different models from fighting over system
+    resources.
+
+    :param num_cores: The number of physical cores to run the model on. If more
+        cores are requested than are available on a single socket, the engine
+        will try to distribute them evenly across as few sockets as possible.
+    :param num_streams: The max number of requests the model can handle
+        concurrently.
+    """
+
+    def __init__(
+        self,
+        num_cores: int = None,
+        num_streams: int = None,
+    ):
+        self._num_cores = _validate_num_cores(num_cores)
+        self._num_streams = _validate_num_streams(num_streams, self._num_cores)
+        self._scheduler = Scheduler.from_str("elastic")
+        self._deepsparse_context = LIB.deepsparse_context(
+            self._num_cores, self._num_streams, self._scheduler.value
+        )
+
+    @property
+    def value(self):
+        return self._deepsparse_context
+
+    @property
+    def num_cores(self):
+        return self._num_cores
+
+    @property
+    def num_streams(self):
+        return self._num_streams
+
+    @property
+    def scheduler(self):
+        return self._scheduler
+
+
+class MultiModelEngine(Engine):
+    """
+    The MultiModelEngine, together with the Context class, can be used to run multiple models
+    on the same computer at once. The interface and behavior are both very similar to the Engine
+    class. The main difference is instead of taking in a scheduler and a number of cores as
+    arguments to the constructor, the MultiModelEngine takes in a Context. The Context contains
+    a shared scheduler along with other runtime information that will be used across instances
+    of the MultiModelEngine to provide optimal performance when running multiple models
+    concurrently.
+
+    :param model: Either a path to the model's onnx file, a SparseZoo model stub
+        prefixed by 'zoo:', a SparseZoo Model object, or a SparseZoo ONNX File
+        object that defines the neural network
+    :param batch_size: The batch size of the inputs to be used with the engine
+    :param context: See above. This object should be constructed with the desired number of
+        cores and passed into each instance of the MultiModelEngine.
+    :param input_shapes: The list of shapes to set the inputs to. Pass None to use model as-is.
+    """
+
+    def __init__(
+        self,
+        model: Union[str, Model, File],
+        batch_size: int,
+        context: Context,
+        input_shapes: List[List[int]] = None,
+    ):
+        self._model_path = model_to_path(model)
+        self._batch_size = _validate_batch_size(batch_size)
+        self._num_cores = context.num_cores
+        self._num_streams = context.num_streams
+        self._scheduler = _validate_scheduler(context.scheduler)
+        self._input_shapes = input_shapes
+        self._cpu_avx_type = AVX_TYPE
+        self._cpu_vnni = VNNI
+
+        if self._input_shapes:
+            with override_onnx_input_shapes(
+                self._model_path, self._input_shapes
+            ) as model_path:
+                self._eng_net = LIB.deepsparse_engine(
+                    model_path,
+                    self._batch_size,
+                    self._num_cores,
+                    self._num_streams,
+                    self._scheduler.value,
+                    context.value,
+                )
+        else:
+            self._eng_net = LIB.deepsparse_engine(
+                self._model_path,
+                self._batch_size,
+                self._num_cores,
+                self._num_streams,
+                self._scheduler.value,
+                context.value,
+            )
+
+
 def compile_model(
     model: Union[str, Model, File],
     batch_size: int = 1,
     num_cores: int = None,
+    num_streams: int = None,
     scheduler: Scheduler = None,
     input_shapes: List[List[int]] = None,
 ) -> Engine:
@@ -541,6 +681,9 @@ def compile_model(
     :param num_cores: The number of physical cores to run the model on.
         Pass None or 0 to run on the max number of cores for the current
         machine; default None
+    :param num_streams: The max number of requests the model can handle
+        concurrently. None or 0 implies a scheduler-defined default value;
+        default None
     :param scheduler: The kind of scheduler to execute with. Pass None for the default.
     :param input_shapes: The list of shapes to set the inputs to. Pass None to use model as-is.
     :return: The created Engine after compiling the model
@@ -549,6 +692,7 @@ def compile_model(
         model=model,
         batch_size=batch_size,
         num_cores=num_cores,
+        num_streams=num_streams,
         scheduler=scheduler,
         input_shapes=input_shapes,
     )
@@ -559,6 +703,7 @@ def benchmark_model(
     inp: List[numpy.ndarray],
     batch_size: int = 1,
     num_cores: int = None,
+    num_streams: int = None,
     num_iterations: int = 20,
     num_warmup_iterations: int = 5,
     include_inputs: bool = False,
@@ -582,6 +727,9 @@ def benchmark_model(
     :param num_cores: The number of physical cores to run the model on.
         Pass None or 0 to run on the max number of cores
         for the current machine; default None
+    :param num_streams: The max number of requests the model can handle
+        concurrently. None or 0 implies a scheduler-defined default value;
+        default None
     :param inp: The list of inputs to pass to the engine for benchmarking.
         The expected order is the inputs order as defined in the ONNX graph.
     :param num_iterations: The number of iterations to run benchmarking for.
@@ -602,6 +750,7 @@ def benchmark_model(
         model=model,
         batch_size=batch_size,
         num_cores=num_cores,
+        num_streams=num_streams,
         scheduler=scheduler,
         input_shapes=input_shapes,
     )
