@@ -81,9 +81,11 @@ deepsparse.server \
 
 import logging
 from pathlib import Path
+from typing import List
 
 import click
 
+from deepsparse import Pipeline
 from deepsparse.log import set_logging_level
 from deepsparse.server.asynchronous import execute_async, initialize_aysnc
 from deepsparse.server.config import (
@@ -91,14 +93,13 @@ from deepsparse.server.config import (
     server_config_from_env,
     server_config_to_env,
 )
-from deepsparse.server.pipelines import load_pipelines_definitions
 from deepsparse.server.utils import serializable_response
 from deepsparse.version import version
 
 
 try:
     import uvicorn
-    from fastapi import FastAPI
+    from fastapi import FastAPI, UploadFile
     from starlette.responses import RedirectResponse
 except Exception as err:
     raise ImportError(
@@ -130,8 +131,37 @@ def _add_general_routes(app, config):
 
 
 def _add_pipeline_route(
-    app, pipeline_def, num_models: int, defined_tasks: set, integration: str
+    app,
+    pipeline: Pipeline,
+    num_models: int,
+    defined_tasks: set,
+    integration: str,
 ):
+    def _create_endpoint(endpoint_path: str, from_files: bool = False):
+        # if `from_files` is True, the endpoint expects request to be
+        # `List[UploadFile]` otherwise, the endpoint expect request to
+        # be `pipeline.input_schema`
+        input_schema = List[UploadFile] if from_files else pipeline.input_schema
+
+        @app.post(
+            endpoint_path,
+            response_model=pipeline.output_schema,
+            tags=["prediction"],
+        )
+        async def _predict_func(request: input_schema):
+            if from_files:
+                files = [file.filename for file in request]
+                request = pipeline.input_schema.from_files(files)
+            results = await execute_async(
+                pipeline,
+                request,
+            )
+            return serializable_response(results)
+
+        _LOGGER.info(f"created route {endpoint_path}")
+
+        return _predict_func
+
     path = "/predict"
 
     if integration.lower() == "sagemaker":
@@ -142,28 +172,29 @@ def _add_pipeline_route(
             )
         # required path name for Sagemaker
         path = "/invocations"
-    elif pipeline_def.config.alias:
-        path = f"/predict/{pipeline_def.config.alias}"
+    elif pipeline.alias:
+        path = f"/predict/{pipeline.alias}"
     elif num_models > 1:
-        if pipeline_def.config.task in defined_tasks:
+        if pipeline.task in defined_tasks:
             raise ValueError(
-                f"Multiple tasks defined for {pipeline_def.config.task} and no alias "
-                f"given for {pipeline_def.config}. "
+                f"Multiple tasks defined for {pipeline.task} and no alias "
+                f"given for pipeline with model {pipeline.model_path_orig}. "
                 "Either define an alias or supply a single model for the task"
             )
-        path = f"/predict/{pipeline_def.config.task}"
-        defined_tasks.add(pipeline_def.config.task)
+        path = f"/predict/{pipeline.task}"
+        defined_tasks.add(pipeline.task)
 
-    @app.post(
-        path,
-        response_model=pipeline_def.response_model,
-        tags=["prediction"],
-    )
-    async def _predict_func(request: pipeline_def.request_model):
-        results = await execute_async(
-            pipeline_def.pipeline, **vars(request), **pipeline_def.kwargs
-        )
-        return serializable_response(results)
+    if hasattr(pipeline.input_schema, "from_files"):
+        if integration.lower() == "sagemaker":
+            # SageMaker supports one endpoint per model, using file upload path
+            _create_endpoint(path, from_files=True)
+        else:
+            # create endpoint for json and file input
+            _create_endpoint(path)
+            _create_endpoint(path + "/from_files", from_files=True)
+    else:
+        # create endpoint with no file support
+        _create_endpoint(path)
 
 
 def server_app_factory():
@@ -183,15 +214,12 @@ def server_app_factory():
     _LOGGER.debug("loaded server config %s", config)
     _add_general_routes(app, config)
 
-    pipeline_defs = load_pipelines_definitions(config)
-    _LOGGER.debug("loaded pipeline definitions from config %s", pipeline_defs)
+    pipelines = [Pipeline.from_config(model_config) for model_config in config.models]
+    _LOGGER.debug("loaded pipeline definitions from config %s", pipelines)
     num_tasks = len(config.models)
     defined_tasks = set()
-
-    for pipeline_def in pipeline_defs:
-        _add_pipeline_route(
-            app, pipeline_def, num_tasks, defined_tasks, config.integration
-        )
+    for pipeline in pipelines:
+        _add_pipeline_route(app, pipeline, num_tasks, defined_tasks, config.integration)
 
     return app
 
