@@ -53,12 +53,12 @@ python eval_downstream.py \
     --dataset squad
 """
 
-
 import argparse
+import json
 
 from tqdm.auto import tqdm
 
-from deepsparse.transformers import pipeline
+from deepsparse import Pipeline
 
 
 from datasets import load_dataset, load_metric  # isort: skip
@@ -79,28 +79,27 @@ def squad_eval(args):
     squad_metrics = load_metric("squad")
 
     # load QA pipeline
-    question_answer = pipeline(
+    question_answer = Pipeline.create(
         task="question-answering",
         model_path=args.onnx_filepath,
         engine_type=args.engine,
         num_cores=args.num_cores,
-        max_length=args.max_sequence_length,
+        sequence_length=args.max_sequence_length,
     )
-    print(f"Engine info: {question_answer.model}")
+    print(f"Engine info: {question_answer.engine}")
 
-    for idx, sample in enumerate(tqdm(squad)):
+    for idx, sample in _enumerate_progress(squad, args.max_samples):
         pred = question_answer(
             question=sample["question"],
             context=sample["context"],
-            num_spans=1,  # only look at first part of long contexts
         )
 
         squad_metrics.add_batch(
-            predictions=[{"prediction_text": pred["answer"], "id": sample["id"]}],
+            predictions=[{"prediction_text": pred.answer, "id": sample["id"]}],
             references=[{"answers": sample["answers"], "id": sample["id"]}],
         )
 
-        if args.max_samples and idx > args.max_samples:
+        if args.max_samples and idx >= args.max_samples:
             break
 
     return squad_metrics
@@ -114,35 +113,38 @@ def mnli_eval(args):
     mnli_metrics = load_metric("glue", "mnli")
 
     # load pipeline
-    text_classify = pipeline(
+    text_classify = Pipeline.create(
         task="text-classification",
         model_path=args.onnx_filepath,
         engine_type=args.engine,
         num_cores=args.num_cores,
-        max_length=args.max_sequence_length,
+        sequence_length=args.max_sequence_length,
     )
-    print(f"Engine info: {text_classify.model}")
+    print(f"Engine info: {text_classify.engine}")
 
-    label_map = {"entailment": 0, "neutral": 1, "contradiction": 2}
+    try:
+        label_map = _get_label2id(text_classify.config_path)
+    except KeyError:
+        label_map = {"entailment": 0, "neutral": 1, "contradiction": 2}
 
-    for idx, sample in enumerate(tqdm(mnli_matched)):
+    for idx, sample in _enumerate_progress(mnli_matched, args.max_samples):
         pred = text_classify([[sample["premise"], sample["hypothesis"]]])
         mnli_metrics.add_batch(
-            predictions=[label_map.get(pred[0]["label"])],
+            predictions=[label_map.get(pred.labels[0])],
             references=[sample["label"]],
         )
 
-        if args.max_samples and idx > args.max_samples:
+        if args.max_samples and idx >= args.max_samples:
             break
 
-    for idx, sample in enumerate(tqdm(mnli_mismatched)):
+    for idx, sample in _enumerate_progress(mnli_mismatched, args.max_samples):
         pred = text_classify([[sample["premise"], sample["hypothesis"]]])
         mnli_metrics.add_batch(
-            predictions=[label_map.get(pred[0]["label"])],
+            predictions=[label_map.get(pred.labels[0])],
             references=[sample["label"]],
         )
 
-        if args.max_samples and idx > args.max_samples:
+        if args.max_samples and idx >= args.max_samples:
             break
 
     return mnli_metrics
@@ -154,26 +156,29 @@ def qqp_eval(args):
     qqp_metrics = load_metric("glue", "qqp")
 
     # load pipeline
-    text_classify = pipeline(
+    text_classify = Pipeline.create(
         task="text-classification",
         model_path=args.onnx_filepath,
         engine_type=args.engine,
         num_cores=args.num_cores,
-        max_length=args.max_sequence_length,
+        sequence_length=args.max_sequence_length,
     )
-    print(f"Engine info: {text_classify.model}")
+    print(f"Engine info: {text_classify.engine}")
 
-    label_map = {"not_duplicate": 0, "duplicate": 1}
+    try:
+        label_map = _get_label2id(text_classify.config_path)
+    except KeyError:
+        label_map = {"not_duplicate": 0, "duplicate": 1, "LABEL_0": 0, "LABEL_1": 1}
 
-    for idx, sample in enumerate(tqdm(qqp)):
+    for idx, sample in _enumerate_progress(qqp, args.max_samples):
         pred = text_classify([[sample["question1"], sample["question2"]]])
 
         qqp_metrics.add_batch(
-            predictions=[label_map.get(pred[0]["label"])],
+            predictions=[label_map.get(pred.labels[0])],
             references=[sample["label"]],
         )
 
-        if args.max_samples and idx > args.max_samples:
+        if args.max_samples and idx >= args.max_samples:
             break
 
     return qqp_metrics
@@ -185,33 +190,47 @@ def sst2_eval(args):
     sst2_metrics = load_metric("glue", "sst2")
 
     # load pipeline
-    text_classify = pipeline(
+    text_classify = Pipeline.create(
         task="text-classification",
         model_path=args.onnx_filepath,
         engine_type=args.engine,
         num_cores=args.num_cores,
-        max_length=args.max_sequence_length,
+        sequence_length=args.max_sequence_length,
     )
-    print(f"Engine info: {text_classify.model}")
+    print(f"Engine info: {text_classify.engine}")
 
-    label_map = {"negative": 0, "positive": 1}
+    try:
+        label_map = _get_label2id(text_classify.config_path)
+    except KeyError:
+        label_map = {"negative": 0, "positive": 1, "LABEL_0": 0, "LABEL_1": 1}
 
-    for idx, sample in enumerate(tqdm(sst2)):
+    for idx, sample in _enumerate_progress(sst2, args.max_samples):
         pred = text_classify(
             sample["sentence"],
         )
 
         sst2_metrics.add_batch(
-            predictions=[label_map.get(pred[0]["label"])],
+            predictions=[label_map.get(pred.labels[0])],
             references=[sample["label"]],
         )
 
-        if args.max_samples and idx > args.max_samples:
+        if args.max_samples and idx >= args.max_samples:
             break
 
     return sst2_metrics
 
 
+def _enumerate_progress(dataset, max_steps):
+    progress_bar = tqdm(dataset, total=max_steps) if max_steps else tqdm(dataset)
+    return enumerate(progress_bar)
+
+
+def _get_label2id(config_file_path):
+    with open(config_file_path) as f:
+        config = json.load(f)
+    return config["label2id"]
+
+
 # Register all the supported downstream datasets here
 SUPPORTED_DATASETS = {
     "squad": squad_eval,
